﻿#ifndef FREETYPEWIDGET_H
#define FREETYPEWIDGET_H

#include <QWidget>
#include <string>
#include "ft2build.h"
#include FT_FREETYPE_H

#include FT_GLYPH_H
#include FT_TYPES_H
#include FT_OUTLINE_H
#include FT_RENDER_H

//#include <QPainter>

class Grid;
class QGraphicsView;

class FreeTypeWidget final : public QWidget
{
    Q_OBJECT
public:
    explicit FreeTypeWidget(QWidget * parent = Q_NULLPTR);
    ~FreeTypeWidget();

    void set_font(const QString & fontPath, uint font_size);
    void set_string(const std::wstring & str, uint manual_height);

    virtual void paintEvent(QPaintEvent * event) override;
    virtual void resizeEvent(QResizeEvent *event) override;

signals:
    void grid_pixel_changed(uint width, uint height);

private:
    void init_widget();
    void calculate_string_box();
    void render_string(int x, int y);
    void draw_bitmap(FT_Bitmap * bitmap, FT_Int x, FT_Int y);

private:
    static constexpr uint border_top = 2;
    FT_Library m_library;
    FT_Face m_face;
    FT_BBox m_bbox;

    std::wstring m_sentence;
    bool m_init_finish = false;
    uint m_manual_height = 0;

    QGraphicsView * m_graphics_view;
    Grid * m_grid;
};

#endif // FREETYPEWIDGET_H
